//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Group/SelectionProperty.h
//! @brief     Defines class SelectionProperty
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_GROUP_SELECTIONPROPERTY_H
#define BORNAGAIN_GUI_MODEL_GROUP_SELECTIONPROPERTY_H

#include "GUI/Model/Group/SelectionDescriptor.h"

// #baMigration docu
template <typename T> class SelectionProperty {
public:
    ~SelectionProperty() { delete m_p; }

    template <typename Catalog>
    void init(const QString& label, const QString& tooltip, const QString& persistentTag)
    {
        m_persistentTag = persistentTag;
        m_descriptor.template init<Catalog>(label, tooltip, &m_p);
        m_p = Catalog::create(Catalog::types()[0]);
    }

    SelectionDescriptor<T> descriptor() const { return m_descriptor; }

    operator SelectionDescriptor<T>() const { return m_descriptor; }
    T operator->() const { return m_p; }

    void set(T t)
    {
        delete m_p;
        m_p = t;
    }

    T get() const { return m_p; }

    QString persistentTag() const { return m_persistentTag; }

private:
    QString m_persistentTag;
    SelectionDescriptor<T> m_descriptor;

    T m_p = nullptr;
};


#endif // BORNAGAIN_GUI_MODEL_GROUP_SELECTIONPROPERTY_H
