//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/FormFactorPlatonicOctahedron.cpp
//! @brief     Implements class FormFactorPlatonicOctahedron.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/FormFactorPlatonicOctahedron.h"
#include "Base/Math/Constants.h"
#include "Base/Math/Functions.h"

const ff::PolyhedralTopology FormFactorPlatonicOctahedron::topology = {{
                                                                   {{0, 2, 1}, false},
                                                                   {{0, 3, 2}, false},
                                                                   {{0, 4, 3}, false},
                                                                   {{0, 1, 4}, false},
                                                                   {{2, 3, 5}, false},
                                                                   {{1, 2, 5}, false},
                                                                   {{4, 1, 5}, false},
                                                                   {{3, 4, 5}, false} },
                                                                  true};

FormFactorPlatonicOctahedron::FormFactorPlatonicOctahedron(const std::vector<double> P)
    : IFormFactorPolyhedron(P)
    , m_edge(m_P[0])
{
    checkNodeArgs();

    const double a = m_edge/2;
    const double h = a * sqrt(2);

    setPolyhedron(topology, -h,
                  {// base:
                   {0, 0, -h},
                   // middle
                   {-a, -a, 0},
                   {a, -a, 0},
                   {a, a, 0},
                   {-a, a, 0},
                   // top
                   {0, 0, h}});
}

FormFactorPlatonicOctahedron::FormFactorPlatonicOctahedron(double edge)
    : FormFactorPlatonicOctahedron(std::vector<double>{edge})
{
}
