//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/FormFactorPlatonicTetrahedron.cpp
//! @brief     Implements class FormFactorPlatonicTetrahedron.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/HardParticle/FormFactorPlatonicTetrahedron.h"
#include "Base/Math/Constants.h"
#include "Base/Math/Functions.h"

const ff::PolyhedralTopology FormFactorPlatonicTetrahedron::topology = {{{{2, 1, 0}, false},
                                                                         {{0, 1, 3}, false},
                                                                         {{1, 2, 3}, false},
                                                                         {{2, 0, 3}, false}},
                                                                        false};

FormFactorPlatonicTetrahedron::FormFactorPlatonicTetrahedron(const std::vector<double> P)
    : IFormFactorPolyhedron(P), m_edge(m_P[0])
{
    checkNodeArgs();

    double a = m_edge;
    double as = a / 2;
    double ac = a / sqrt(3) / 2;
    double ah = a / sqrt(3);

    double zcom = height() / 4; // center of mass

    setPolyhedron(topology, -zcom,
                  {// base:
                   {-ac, as, -zcom},
                   {-ac, -as, -zcom},
                   {ah, 0., -zcom},
                   // top:
                   {0, 0., height() - zcom}});
}

FormFactorPlatonicTetrahedron::FormFactorPlatonicTetrahedron(double edge)
    : FormFactorPlatonicTetrahedron(std::vector<double>{edge})
{
}
