//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/IFormFactorPrism.h
//! @brief     Defines interface IFormFactorPrism.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef USER_API
#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPRISM_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPRISM_H

#include "Sample/Particle/IBornFF.h"
#include <memory>

namespace ff {

class Prism;

} // namespace ff


//! A prism with a polygonal base, for form factor computation.

class IFormFactorPrism : public IBornFF {
public:
    IFormFactorPrism(const std::vector<double>& PValues);
    ~IFormFactorPrism() override;

    double bottomZ(const IRotation* rotation) const override;
    double topZ(const IRotation* rotation) const override;

    complex_t formfactor_at_bottom(C3 q) const override;
    double volume() const override;
    double radialExtension() const override;
    virtual double height() const = 0; // TODO mv parameter m_height back from children to this

protected:
    void setPrism(bool symmetry_Ci, const std::vector<R3>& vertices);

private:
    std::unique_ptr<ff::Prism> pimpl;
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_IFORMFACTORPRISM_H
#endif // USER_API
